import { hashToClassName } from './util';

var selfClosing = ['area', 'base', 'br', 'col', 'command', 'embed', 'hr', 'img', 'input', 'keygen', 'link', 'meta', 'param', 'source', 'track', 'wbr'];

export default function h(tag, attrs) {
  for (var _len = arguments.length, children = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  if (typeof tag === 'function') {
    if (tag.prototype && tag.prototype.constructor && tag.prototype.render) {
      // eslint-disable-next-line new-cap
      var instance = new tag(Object.assign({}, attrs, { children: children }));
      return instance.render(instance.props, instance.state) || '';
    }

    return tag(Object.assign({}, attrs, { children: children }));
  }

  if (typeof tag !== 'string') {
    // so that we can use h() to convert html string
    // into node like <div>{h(props.region)}</div>
    return null;
  }

  var str = '<' + tag;

  if (attrs) {
    Object.keys(attrs).forEach(function (key) {
      var value = attrs[key];

      if (value != null && !(value instanceof Function)) {
        if (key === 'className') key = 'class';

        if (key === 'class') {
          if (Array.isArray(value)) {
            value = value.filter(function (value) {
              return value;
            }).join(' ');
          } else if ((typeof value === 'undefined' ? 'undefined' : babelHelpers.typeof(value)) === 'object') {
            value = hashToClassName(value);
          }
        }

        str += ' ' + key + '="' + value + '"';
      }
    });
  }

  if (selfClosing.includes(tag)) {
    str += '/>';
    return str;
  }

  str += '>';

  while (children.length) {
    var child = children.shift();

    if (Array.isArray(child)) {
      children.unshift.apply(children, babelHelpers.toConsumableArray(child));
    } else if (child != null && child !== false) {
      str += child;
    }
  }

  return str + '</' + tag + '>';
}