'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _util = require('./util');

function appendChildren(el, children) {
  children.forEach(function (child) {
    if (child instanceof Node) {
      el.appendChild(child);
    } else if (typeof child === 'string' || typeof child === 'number') {
      el.appendChild(document.createTextNode(child));
    } else if (Array.isArray(child)) {
      appendChildren(el, child);
    }
  });
}

exports.default = function (tag, attrs) {
  for (var _len = arguments.length, children = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    children[_key - 2] = arguments[_key];
  }

  if (typeof tag === 'function') {
    return tag(Object.assign({}, attrs, { children: children }));
  }

  if (typeof tag !== 'string') {
    // so that we can use h() to convert html string
    // into node like <div>{h(props.region)}</div>
    return null;
  }

  if (tag.startsWith('<')) {
    // TODO make sure this is valid HTML
    var container = document.createElement('div');

    container.innerHTML = tag;

    return Array.from(container.childNodes);
  }

  var el = document.createElement(tag);

  appendChildren(el, children);

  if (attrs) {
    Object.keys(attrs).forEach(function (key) {
      var value = attrs[key];

      if (!value && value !== 0) return;

      if (key === 'className') key = 'class';

      if (key === 'class') {
        if (Array.isArray(value)) {
          value = value.filter(function (value) {
            return value;
          }).join(' ');
        } else if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object') {
          value = (0, _util.hashToClassName)(value);
        }
      }

      el.setAttribute(key, value);
    });
  }

  return el;
};

module.exports = exports['default'];